using UnityEngine;

/// <summary>
/// Line renderer that mixes always-on-top lines with depth-tested lines.
/// </summary>

namespace HAL9001
{
	public class DebugLineRenderer
	{
		public DebugLineRenderer(string _gameObjectName, int _pointCount = 0)
		{
			m_gameObjectDepth = new GameObject(_gameObjectName + "_DepthTest");
			m_gameObjectTop   = new GameObject(_gameObjectName + "_Top");

			m_rendererDepth = m_gameObjectDepth.AddComponent<LineRenderer>();
			m_rendererDepth.useWorldSpace        = true;
			m_rendererDepth.material             = new Material(Shader.Find("Sprites/Default"));
			m_rendererDepth.material.renderQueue = 4000;

			m_rendererTop = m_gameObjectTop.AddComponent<LineRenderer>();
			m_rendererTop.useWorldSpace        = true;
			m_rendererTop.material             = new Material(Shader.Find("GUI/Text Shader"));
			m_rendererTop.material.renderQueue = 4000; 

			SetPointCount(_pointCount);
			FinalisePoints();
			SetWidth(0.01f);
			SetColor(Color.red);
		}

		
		public void SetPointCount(int idx)
		{
			m_points = new Vector3[idx];
		}


		public void SetPoint(int _idx, float _x, float _y, float _z)
		{
			if ((_idx >=0) && (_idx < m_points.Length))
			{
				m_points[_idx].Set(_x, _y, _z);
			}
		}


		public void SetPoint(int _idx, Vector3 _pos)
		{
			if ((_idx >=0) && (_idx < m_points.Length))
			{
				m_points[_idx] = _pos;
			}
		}


		public void FinalisePoints()
		{
			m_rendererDepth.positionCount = m_points.Length;
			m_rendererTop.positionCount   = m_points.Length;
			m_rendererDepth.SetPositions(m_points);
			m_rendererTop.SetPositions(m_points);
		}


		public void SetWidth(float _width)
		{
			m_rendererDepth.widthMultiplier = _width;
			m_rendererTop.widthMultiplier   = _width / 2;
		}


		public void SetColor(Color _col)
		{
			Color colTop = _col;
			colTop.a = 0.25f;
			m_rendererDepth.startColor = _col;
			m_rendererTop.startColor   = colTop;
			m_rendererDepth.endColor   = _col;
			m_rendererTop.endColor     = colTop;
		}


		public void Destroy()
		{
			GameObject.Destroy(m_gameObjectDepth);
			GameObject.Destroy(m_gameObjectTop);
			m_gameObjectDepth = null;
			m_gameObjectTop   = null;
		}


		protected GameObject   m_gameObjectDepth, m_gameObjectTop;
		protected LineRenderer m_rendererDepth, m_rendererTop;
		protected Vector3[]    m_points;
	}
}
